#include "main.h"

// Declaration for an SSD1306 display connected to I2C (SDA, SCL pins)
#define OLED_RESET LED_BUILTIN  //不同PCB開發板得到不同的數值
// 初始化 
#define OLED_RESET     -1 // Reset pin # (or -1 if sharing Arduino reset pin)
#define SIZE_OLED 64  //3 * 10 = 3列，每列10個字
#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 64 // OLED display height, in pixels
#define SCREEN_ADDRESS 0x3C ///< See datasheet for Address; 0x3D or 0x3C
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET); //for Uno
#define LINEMAX 10
char buff_oled[LINEMAX*4] = {' '};
char OLED_OK = false;
char bNeedRefesh = false;
long Speed;

void clear_oled( char lineno ){
  char i,x = (lineno-1)*LINEMAX;
  for ( i=x ; i<(x+LINEMAX); i++){
    buff_oled[i] = ' '; //fill <SPACE>
  }
}

void putch_oled(char lineno, char ch ) { 
  static char x[3]={0,0,0};
  --lineno; // {1,2,3} to {0,1,2}
  if( ch==CR || ch==LF ) {
    char i = 2*LINEMAX;
    if (buff_oled[i]=='T' && buff_oled[i+1]=='X' && buff_oled[i+2]=='=' ) {
      clear_oled(2+1);
    }
    x[lineno] = 0;    //X座標歸零
    if ( ch==CR ) bNeedRefesh = true;
  } else {
    if (x[lineno]==0) clear_oled(lineno+1); //clear first
    buff_oled[ (lineno)*LINEMAX + x[lineno]] = ch;
    ++x[lineno]; //point to the next buffer
  }
}
void print_oled(char lineno, char *str ) { 
  char i = 0, j = 0;
  while( i < LINEMAX ) {
    if ( str[j] ) {
      buff_oled[ (lineno-1)*LINEMAX + i++ ] = str[j++];
    } else {
      buff_oled[ (lineno-1)*LINEMAX + i++ ] = ' ';
    }
    bNeedRefesh = true;
  }
}

void refresh_OLED( void ) {
  static long iOT = millis();
  static char inited = false;
  if ( inited == false ) {
      if(display.begin(SSD1306_SWITCHCAPVCC, SCREEN_ADDRESS)) OLED_OK = true;
      inited = true;
  }
  if ( millis() - iOT > 10 ) {
    if ( bNeedRefesh == true ){   
      //digitalWrite( D0, LOW ); 
      //Speed = millis();
      display.clearDisplay();
      display.setTextSize(2);      // Normal 1:1 pixel scale
      display.setTextColor(WHITE); // Draw white text
      display.cp437(true);         // Use full 256 char 'Code Page 437' font  
      display.setCursor(0, 0);     // Start at top-left corner ( x, y )
      for(char i=0; i<LINEMAX*1; i++) display.write( buff_oled[i] );
      display.setCursor(0, 21);     // Start at top-left corner
      for(char i=LINEMAX*1; i<LINEMAX*2; i++) display.write( buff_oled[i] );
      display.setCursor(0, 42);     // Start at top-left corner
      for(char i=LINEMAX*2; i<LINEMAX*3; i++) display.write( buff_oled[i] );
      display.display();  //
      //Speed = iOT - Speed;
      //digitalWrite( D0, HIGH );
      bNeedRefesh = false;
      iOT = millis();
    }
  }
}
